import json
import urllib.request as urlreq
from random import randint
from typing import *
import xml.etree.ElementTree as ET

class Safebooru:
    def __init__(self):
        self.page_num = randint(0, 19)
        self.booru_url = 'https://safebooru.org/index.php?page=dapi&s=post&q=index&json=1'
        self.comment_url = 'https://safebooru.org/index.php?page=dapi&s=comment&q=index'

    # Private function to fix tags so that they work with the image board
    def __tagifier(self, unformated_tags):
        fixed_tags = unformated_tags.replace(', ', r'%20').replace(' ', '_').lower()
        return fixed_tags

    # Returns up too 100 posts and images based on tags the user inputs
    def get_posts(self, tags='', limit=100):
        if limit > 100:
            raise Exception("Limit cannot be greater than 100")

        tags = self.__tagifier(tags)

        try:
            final_url = self.booru_url + f'&tags={tags}&limit={limit}&pid={self.page_num}'
            urlobj = urlreq.urlopen(final_url)
            json_response = json.load(urlobj)
            urlobj.close()
        except:
            return None

        temp = 4
        while len(json_response) == 0:
            self.page_num = randint(0, temp)

            if temp > 0:
                temp += -1
            else:
                pass
            try:
                final_url = self.booru_url + f'&tags={tags}&limit={limit}&pid={self.page_num}'
                urlobj = urlreq.urlopen(final_url)
                json_response = json.load(urlobj)
                urlobj.close()
            except:
                return None
            temp = 4
        
        images = self.__link_images(json_response)
        return images

    # Return a single post and image based on tags the user inputs
    def get_single_post(self, tags=''):
        tags = self.__tagifier(tags)
        final_url = self.booru_url + f'&tags={tags}&limit=1&pid={self.page_num}'
        
        try:
            urlobj = urlreq.urlopen(final_url)
            json_response = json.load(urlobj)
            urlobj.close()
        except:
            return None

        temp = 4

        # Reduce search if json_response is empty
        while len(json_response) == 0:
            self.page_num = randint(0, temp)
            
            if temp > 0:
                temp += -1 # Further reduction
            else:
                pass
            final_url = self.booru_url + f'&tags={tags}&limit={1}&pid={self.page_num}'
            try:
                urlobj = urlreq.urlopen(final_url)
                json_response = json.load(urlobj)
                urlobj.close()
            except:
                return None
            temp = 4

        image = self.__link_images(json_response)
        return image

    #Get a random post. This selects a single image out of 20,100 images.
    def get_random_post(self):
        self.page_num = randint(0, 200)
        final_url = self.booru_url + f'&limit={100}&pid={self.page_num}'
        urlobj = urlreq.urlopen(final_url)
        json_response = json.load(urlobj)
        urlobj.close()

        temp = [json_response[randint(0,99)]]
        image = self.__link_images(temp)
        return image

    # Get data from a post
    def get_post_data(self, post_id):
        data_url = f'https://safebooru.org/index.php?page=dapi&s=post&q=index&id={post_id}'

        urlobj = urlreq.urlopen(data_url)
        data = ET.parse(urlobj)
        urlobj.close()
        root = data.getroot()

        return root[0].attrib # Returns a dictionary

    # Private function to link images from the various functions above.
    def __link_images(self, response):
        image_list = []
        temp_dict = dict()
        temp = 1
        post_url = 'https://safebooru.org/index.php?page=post&s=view'

        for i in response:
            directory = i['directory']
            post_id = i['id']
            image_ext = i['image']
            image_url = f'https://safebooru.org/images/{directory}/{image_ext}'
            temp_dict[f'post {temp} url'] = post_url + f'&id={post_id}'
            temp_dict[f'image {temp} url'] = image_url
            image_list.append(temp_dict)
            temp += 1
            temp_dict = dict()

        return image_list


import json
import urllib.request as urlreq
from random import randint
from typing import *
import xml.etree.ElementTree as ET

"""
Works with gelbooru API.
ET.ParseError should never happen; tags that dont exist will just return an empty XML and be converted to an empty list.
Using too many tags will result in just a few available images which may end up not working.
"""

class Gelbooru:

    def __init__(self, api_key: Optional[str] = None, user_id: Optional[str] = None):
        self.api_key = api_key
        self.user_id = user_id
        self.page_num = randint(0, 19)
        self.booru_url = 'https://gelbooru.com/index.php?page=dapi&s=post&q=index'
        self.comment_url = 'https://gelbooru.com/index.php?page=dapi&s=comment&q=index'

    def __tagifier(self, unformated_tags):
        fixed_tags = unformated_tags.replace(', ', r'%20').replace(' ', '_').lower()
        return fixed_tags
    
    # Get a bunch of posts based on a limit and tags that the user enters.
    def get_posts(self, tags='', limit=100):
        posts = []
        tags = self.__tagifier(tags)
        final_url = self.booru_url + f'&limit={str(limit)}&tags={tags}&pid={self.page_num}&api_key={self.api_key}&user_id={self.user_id}'
        
        try:
            urlobj = urlreq.urlopen(final_url)
            data = ET.parse(urlobj)
            urlobj.close()
            root = data.getroot()
        except ET.ParseError:
            return None

        for post in root:
            posts.append(post.attrib)
        
        temp = 4
        attempts = 5
        while len(posts) == 0:
            self.page_num = randint(0, temp)
            if temp > 0:
                temp -= 1
            else:
                pass
            final_url = self.booru_url + f'&limit={str(limit)}&tags={tags}&pid={self.page_num}&api_key={self.api_key}&user_id={self.user_id}' 
            try:
                urlobj = urlreq.urlopen(final_url)
                data = ET.parse(urlobj)
                urlobj.close()
                root = data.getroot()
            except ET.ParseError:
                return None

            attempts -= 1
            if attempts == 0:
                return None
            else:
                continue
            for post in root:
                posts.append(post.attrib)
        
        images = self.__link_images(posts)
        return images

    # Get a single image based on tags that the user enters.
    def get_single_post(self, tags=''):
        tags = self.__tagifier(tags)
        posts = []
        final_url = self.booru_url + f'&limit=1&tags={tags}&pid={self.page_num}&api_key={self.api_key}&user_id={self.user_id}'
        try:
            urlobj = urlreq.urlopen(final_url)
            data = ET.parse(urlobj)
            urlobj.close()
            root = data.getroot()
        except ET.ParseError:
            return None
        
        for post in root:
            posts.append(post.attrib)
        
        # reduces search if no results found
        temp = 4
        attempts = 5
        while len(posts) == 0:
            self.page_num = randint(0, temp)
            if temp > 0:
                temp -= 1
            else:
                pass
            final_url = self.booru_url + f'&limit=1&tags={tags}&pid={self.page_num}&api_key={self.api_key}&user_id={self.user_id}' 
            try:
                urlobj = urlreq.urlopen(final_url)
                data = ET.parse(urlobj)
                urlobj.close()
                root = data.getroot()
            except ET.ParseError:
                return None

            attempts -= 1
            if attempts == 0:
                return None
            else:
                continue
        
        image = self.__link_images(posts)
        return image
    
    # Random post picks a random image out of 20000+ images
    def get_random_post(self):
        self.page_num = randint(0, 200)
        posts = []
        final_url = self.booru_url + f'&pid={self.page_num}&api_key={self.api_key}&user_id={self.user_id}'
        
        try:
            urlobj = urlreq.urlopen(final_url)
            data = ET.parse(urlobj)
            urlobj.close()
            root = data.getroot()
        except ET.ParseError:
            return None
        
        posts.append(root[randint(0,99)].attrib)
        image = self.__link_images(posts)
        return image
        
    # Get comments from a post using post_id
    def get_comments(self, post_id):
        comment_list = []
        final_url = self.comment_url + f'&post_id={post_id}&api_key={self.api_key}&user_id={self.user_id}'
        try:
            urlobj = urlreq.urlopen(final_url)
            data = ET.parse(urlobj)
            urlobj.close()
        except:
            return None

        root = data.getroot()
        temp = dict()
        
        # Iterate through comments
        for i in range(len(root)):
            temp['author'] = root[i].attrib['creator']
            temp['comment'] = root[i].attrib['body']
            comment_list.append(temp)
            temp = dict()

        if len(comment_list) == 0:
            return "No comments found"
        else:
            return comment_list
    
    # Get data for a post
    def get_post_data(self, post_id):
        data_url = f'https://gelbooru.com/index.php?page=dapi&s=post&q=index&id={post_id}'
        try:
            urlobj = urlreq.urlopen(data_url)
            data = ET.parse(urlobj)
            urlobj.close()
        except:
            return None

        root = data.getroot()

        return root[0].attrib

    # Private function to create a post URL and a related image URL
    def __link_images(self, response):
        image_list = []
        temp_dict = dict()

        post_url = 'https://gelbooru.com/index.php?page=post&s=view&id='
        for i in range(len(response)):
            temp_dict['post url'] = post_url + f'{response[i]["id"]}'
            temp_dict['image url'] = response[i]['file_url']
            temp_dict['id'] = response[i]['id']
            image_list.append(temp_dict)
            temp_dict = dict()

        return image_list # Returns image URL(s) and post URL(s) in a list


import json
import urllib.request as urlreq
from random import randint
from typing import *
import xml.etree.ElementTree as ET

class Danbooru:
    # Danbooru needs an API key unlike other boorus like Gelbooru and Safebooru
    def __init__(self, api_key, login):
        self.api_key = api_key
        self.login = login
        self.page_num = randint(0, 19)
        self.booru_url = 'https://danbooru.donmai.us/posts.json?'
        self.post_url = 'https://danbooru.donmai.us/posts/'

    def __tagifier(self, unformated_tags):
        fixed_tags = unformated_tags.replace(', ', r'%20').replace(' ', '_').lower()
        return fixed_tags

    # Get posts based on tags the user inputs
    def get_posts(self, tags='', limit=100):
        tags = self.__tagifier(tags)
        final_url = self.booru_url + f'&login={self.login}&api_key={self.api_key}&limit={limit}&tags={tags}'
        urlobj = urlreq.urlopen(final_url)
        json_response = json.load(urlobj)
        urlobj.close()

        if not json_response:
            return None # Return None if no images found

        images = self.__link_images(json_response)
        
        return images

    # Danbooru API has a "random" keyword :D
    def get_random_post(self):
        final_url = self.booru_url + f'&login={self.login}&api_key={self.api_key}&random=true&limit=1'
        
        urlobj = urlreq.urlopen(final_url)
        json_response = json.load(urlobj)
        urlobj.close()
        
        image = self.__link_images(json_response)

        return image

    def get_image_data(self, post_id):
        final_url = self.post_url + f'{post_id}.json'

        try: 
            urlobj = urlreq.urlopen(final_url)
            json_response = json.load(urlobj)
            urlobj.close()
        except:
            return None
        
        return json_response

    # Private function to create dictionaries of posts/images
    def __link_images(self, response):
        image_list = []
        temp_dict = dict()
        temp = 1
        post_url = 'https://danbooru.donmai.us/posts/'

        for i in response:
            post_id = i['id']
            file_url = i['file_url']
            temp_dict[f'post {temp} url'] = post_url + str(post_id)
            temp_dict[f'image {temp} url'] = file_url
            image_list.append(temp_dict)
            temp += 1
            temp_dict = dict()
        
        return image_list